from __future__ import division, with_statement, print_function

import os
from game import Game
from level import Level
from menu import Menu


def get_level_list(game):
    level = game.level
    files = os.listdir('media/levels')
    levels = []
    for file in files:
        if os.path.splitext(file)[1] != '.lvl': continue
        level.load_from_file('media/levels/'+file)
        levels.append(('media/levels/'+file, level.title, level.description, level.hash))
    return sorted(levels)
    
    
def read_score():
    "Read list of completed levels and initial solving time."
    scores = {}
    try:
        with open('media/levels/score.dat', 'r') as f:
            for line in f:
                line = line[:-1]  # get rid of newline
                params = line.split(' ')
                scores[params[0]] = int(params[1])
    except: pass
    return scores
    
    
def write_score(scores):
    "Write list of completed levels and initial solving time."
    with open('media/levels/score.dat', 'w') as f:
        for hash in scores:
            f.write("{0} {1}\n".format(hash, scores[hash]))
    
    
try:
    # initialise
    game = Game()
    game.initialise()
    
    # get level list
    levels = get_level_list(game)
    scores = read_score()
    
    # create menu
    menu = Menu(game, levels, scores)
    menu.run()
    # since scores is a reference, can save back directly
    write_score(scores)
    
finally:
    game.shutdown()
