from __future__ import division, with_statement, print_function

import pygame
from interface import Button
from game import Game

FREE_SELECT = False

class Menu:
    def __init__(self, game, levels, scores):
        self.game = game
        self.display = self.game._display
        self.levels = levels
        self.scores = scores
        self.curLevel = 0
        # select first unsolved level, if available
        while self.curLevel < len(self.levels)-1 and self.levels[self.curLevel][3] in self.scores:
            self.curLevel += 1
        self.render_level_details()
        self._continue = True
        self.next = Button('media/gui/button_right.png', (910,320))
        self.prev = Button('media/gui/button_left.png', (50,320))
        self.start = Button('media/gui/button_start.png', (448, 680))
        self.background = pygame.image.load('media/gui/main_background.png')
        
        
    def render_level_details(self):
        self.levelLocked = (self.curLevel > 0 and self.levels[self.curLevel-1][3] not in self.scores) and not FREE_SELECT
        level = self.levels[self.curLevel]
        colour = (170,0,32)
        if self.levelLocked:
            colour = (128,128,128)
        descWidth = 724
        titleFont = pygame.font.Font('media/freesansbold.ttf', 60)
        extraFont = pygame.font.Font('media/freesansbold.ttf', 30)
        self.title = titleFont.render("{0}: {1}".format(self.curLevel+1, level[1]), True, colour)
        self.descLines = []
        words = level[2].split(' ')
        line = ''
        for word in words:
            if line == '':
                line = word
                continue
            newline = line + ' ' + word
            if extraFont.size(newline)[0] > descWidth:
                self.descLines.append(extraFont.render(line, True, colour))
                line = word
            else:
                line = newline
        if line != '':
            self.descLines.append(extraFont.render(line, True, colour))
        # determine extra info line
        if self.levelLocked:
            self.extra = extraFont.render("Complete the previous level first", True, colour)
        elif self.levels[self.curLevel][3] not in self.scores:
            self.extra = extraFont.render("Unsolved", True, colour)
        else:
            score = int(self.scores[self.levels[self.curLevel][3]])
            minutes = score // 60
            seconds = score % 60
            self.extra = extraFont.render("First solved in {0}:{1:02}".format(minutes,seconds), True, colour)
            
            
            
    def next_level(self):
        self.curLevel = (self.curLevel+1) % len(self.levels)
        self.render_level_details()


    def prev_level(self):
        self.curLevel = (self.curLevel-1) % len(self.levels)
        self.render_level_details()
        
        
    def run(self):
        clock = pygame.time.Clock()
        clock.tick()
        self.setMusic = False
        self.play_music()
        
        while self._continue:
            clock.tick(120)
            for event in pygame.event.get():
                self.process_event(event)
            self.update(clock.get_time())
            self.render()
            
            
    def render(self):
        #self.display.fill(Menu.BACKGROUND)
        self.display.blit(self.background, (0,0))
        self.next.render(self.display)
        self.prev.render(self.display)
        if not self.levelLocked:
            self.start.render(self.display)
        # display title and description
        self.display.blit(self.title, (512 - self.title.get_width()//2, 335))
        x = 150
        y = 420
        for desc in self.descLines:
            self.display.blit(desc, (512 - desc.get_width()//2, y))
            y += desc.get_height() + 5
        self.display.blit(self.extra, (512 - self.extra.get_width()//2, 640))
            
        pygame.display.flip()
        
        
    def update(self, time):
        pass
        
        
    def play_music(self):
        if self.setMusic:
            self.setMusic = False
            return
        # play menu song
        file = 'media/sound/menu.ogg'
        if not pygame.mixer.music.get_busy():
            pygame.mixer.music.load(file)
            pygame.mixer.music.play()
        else:
            self.setMusic = True
            pygame.mixer.music.fadeout(1000)
            pygame.mixer.music.queue(file)
        pygame.mixer.music.set_endevent(Game.MUSIC_ENDED)
        
        
    def process_event(self, event):
        if event.type == Game.MUSIC_ENDED:
            self.play_music()
        elif event.type == pygame.QUIT:
            self._continue = False
        elif event.type == pygame.MOUSEBUTTONDOWN:
            if self.next.is_clicked(event.pos[0], event.pos[1]):
                self.next_level()
            elif self.prev.is_clicked(event.pos[0], event.pos[1]):
                self.prev_level()
            elif self.start.is_clicked(event.pos[0], event.pos[1]) and not self.levelLocked:
                won = self.game.run(self.levels[self.curLevel][0])
                self.setMusic = False
                self.play_music()
                if won and not self.levels[self.curLevel][3] in self.scores:
                    self.scores[self.levels[self.curLevel][3]] = int(self.game.timeElapsed)
                    self.curLevel += 1
                    if self.curLevel >= len(self.levels):
                        self.curLevel = len(self.levels)-1
                    self.render_level_details()
    
